/*****************************************************************************
 *  Rohde&Schwarz FSH instrument driver
 *  Callbacks
 *  Original Release: July 2016
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *     see ChangeLog
 *
 *****************************************************************************/

#include "rsfph.h"
#include "rsfph_utility.h"
#include <float.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
    #define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
        #include <windows.h>
#endif

#pragma warning (disable:4100 4055) // unreferenced formal parameter

/*****************************************************************************
 *- Read Callbacks ----------------------------------------------------------*
 *****************************************************************************/
/*****************************************************************************
 * Function: rsfph_ReadOnlyCollection_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Returns value
 *           from cache
 *****************************************************************************/
ViStatus rsfph_ReadOnlyCollection_ReadCallback(ViSession          instrSession,
                                               ViConstString      repCapName,
                                               RsCoreAttributePtr attr)
{
    // this callback is necessary only in IVI.NET, in C it's standard ViString callback
    return RsCore_ReadCallback(instrSession, repCapName, attr);
}

/*****************************************************************************
 *- Write Callbacks ---------------------------------------------------------*
 *****************************************************************************/
/*****************************************************************************
 * Function: rsfph_FileStateSpecialFormat_WriteCallback
 * Purpose:  This function formats 1, before the argument string
 *****************************************************************************/
ViStatus rsfph_FileStateSpecialFormat_WriteCallback(ViSession          instrSession,
                                                    ViConstString      repCapName,
                                                    RsCoreAttributePtr attr,
                                                    void               *value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " 1,'", value, "'");
}

/*****************************************************************************
 * Function: rsscope_logging_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Returns value
 *           from cache
 *****************************************************************************/
ViStatus rsfph_logging_ReadCallback(ViSession          instrSession,
                                    ViConstString      repCapName,
                                    RsCoreAttributePtr attr)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: rsspecan_logging_WriteCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rsfph_logging_WriteCallback(ViSession          instrSession,
                                     ViConstString      repCapName,
                                     RsCoreAttributePtr attr,
                                     void               *value)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: rsfph_quotedInt_WriteCallback
 * Purpose:  This function overrides standard calback funtion. Writes int
 *           with apostrophes
 *****************************************************************************/
ViStatus rsfph_quotedInt_WriteCallback(ViSession          instrSession,
                                       ViConstString      repCapName,
                                       RsCoreAttributePtr attr,
                                       void               *value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " '", value, "'");
}
