/*****************************************************************************
 *------------------------ Utility  -----------------------------------------*
 *****************************************************************************/
#include <float.h>
#include "rsfph.h"

#pragma warning(disable:4152)

/*****************************************************************************
 * Function: rsfph_InitAttributes
 * Purpose:  This function inits attributes to the desired values if needed.
 *****************************************************************************/
ViStatus rsfph_InitAttributes(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	// - Driver Identification -
	checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 0, RSFPH_SPECIFIC_DRIVER_DESCRIPTION));
	checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_PREFIX, 0, RSFPH_SPECIFIC_DRIVER_PREFIX));
	checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_LOCATOR, 0, RSFPH_SPECIFIC_DRIVER_LOCATOR));
	checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_VENDOR, 0, RSFPH_SPECIFIC_DRIVER_VENDOR));
	checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, RSFPH_CLASS_SPEC_MAJOR_VERSION));
	checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, RSFPH_CLASS_SPEC_MINOR_VERSION));

	// - Driver Capabilities -
	checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SUPPORTED_INSTRUMENT_MODELS, 0, RSFPH_SUPPORTED_INSTRUMENT_MODELS));
	checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_GROUP_CAPABILITIES, 0, RSFPH_GROUP_CAPABILITIES));
	checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_FUNCTION_CAPABILITIES, 0, RSFPH_FUNCTION_CAPABILITIES));

	// - Version Info -
	checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION, 0, RSFPH_MAJOR_VERSION));
	checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_VERSION, 0, RSFPH_MINOR_VERSION));
	checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_MINOR_VERSION, 0, RSFPH_MINOR_MINOR_VERSION));

	checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MAJOR_VERSION, 0, RSFPH_CLASS_SPEC_MAJOR_VERSION));
	checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MINOR_VERSION, 0, RSFPH_CLASS_SPEC_MINOR_VERSION));

	checkErr(RsCore_SetSpecificDriverRevision(instrSession, RS_ATTR_SPECIFIC_DRIVER_REVISION));

Error:
	return error;
}

/*****************************************************************************
 * Function: rsfph_DefaultInstrSetup
 * Purpose:  This function sends a default setup to the instrument. The
 *           rsfph_reset function calls this function. The
 *           rsfph_RsInit function calls this function when the
 *           user passes VI_FALSE for the reset parameter. This function is
 *           useful for configuring settings that other instrument driver
 *           functions require.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsfph_DefaultInstrSetup(ViSession instrSession)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));

    // Set all the attributes to the default state. Do not update inherent attributes!
    checkErr(RsCore_ApplyAttributeDefaults(instrSession, VI_FALSE));

    // Init attributes
    checkErr(rsfph_InitAttributes(instrSession));

    rsSession->fastSweepInstrument = 0;
    checkErr(RsCore_ResetRegistersEseSre(instrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsfph_CheckStatus
 * Purpose:  This function checks the status of the instrument to detect
 *           whether the instrument has encountered an error. This function
 *           is called at the end of most exported driver functions. If the
 *           instrument reports an error, this function returns
 *           RS_ERROR_INSTRUMENT_SPECIFIC. The user can set the
 *           rsfph_ATTR_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE to disable this
 *           check and increase execution speed.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsfph_CheckStatus(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	if (RsCore_QueryInstrStatus(instrSession) && !RsCore_Simulating(instrSession))
	{
		checkErr(RsCore_CheckStatus(instrSession, VI_SUCCESS));
	}

Error:
	return error;
}

/****************************************************************************************************
 *  rsfph_dataReadTrace
	Queries Float array in BIN or ASCII format (sets BIN format, but can handle both)
	You have to provide an allocated traceData[] buffer
******************************************************************************************************/
ViStatus rsfph_dataReadTrace(ViSession instrSession,
                             ViString  cmd,
                             ViInt32   arrayLength,
                             ViReal64  traceData[],
                             ViPInt32  noofPoints)
{  
	ViStatus error = VI_SUCCESS;
	ViChar   fullCmd[RS_MAX_MESSAGE_BUF_SIZE];

	snprintf(fullCmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;%s", cmd);
	checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, fullCmd, arrayLength, traceData, noofPoints));

Error:
	return error;
}

ViStatus rsfph_utility_error_message(ViSession instrSession,
                                      ViStatus  errorCode,
                                      ViChar    errorMessage[])
{
	ViStatus error = VI_SUCCESS;
	static   RsCoreStringValueTable errorTable =
	         {
	         RSFPH_ERROR_CODES_AND_MSGS,
	{0,      NULL}
	         };

	(void)(RsCore_LockSession(instrSession));

	// all VISA and RS error codes are handled as well as codes in the table
	viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

	checkErr(RsCore_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, 256, errorTable));

Error:  
	(void)RsCore_UnlockSession(instrSession); // IGNORE: ERROR!!! Missing Lock
	return error;
}